/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     ContextTables.h
    \brief    Defines constants and tables for SBAC
    \todo     number of context models is not matched to actual use, should be fixed
*/

#ifndef __CONTEXTTABLES__
#define __CONTEXTTABLES__

//! \ingroup TLibCommon
//! \{

// ====================================================================================================================
// Constants
// ====================================================================================================================

#define MAX_NUM_CTX_MOD             512       ///< maximum number of supported contexts

#define NUM_SPLIT_FLAG_CTX            3       ///< number of context models for split flag
#define NUM_SKIP_FLAG_CTX             3       ///< number of context models for skip flag

#if DNB_MERGE_FLAG
#define NUM_MERGE_FLAG_EXT_CTX        1       ///< number of context models for merge flag of merge extended
#else
#define NUM_MERGE_FLAG_EXT_CTX        3       ///< number of context models for merge flag of merge extended
#endif
#define NUM_MERGE_IDX_EXT_CTX         4       ///< number of context models for merge index of merge extended

#if DNB_ALF_CTRL_FLAG
#define NUM_ALF_CTRL_FLAG_CTX         1       ///< number of context models for ALF control flag
#else
#define NUM_ALF_CTRL_FLAG_CTX         3       ///< number of context models for ALF control flag
#endif
#define NUM_PART_SIZE_CTX             5       ///< number of context models for partition size
#if AMP
#define NUM_CU_X_POS_CTX              2       ///< number of context models for partition size (AMP)
#define NUM_CU_Y_POS_CTX              2       ///< number of context models for partition size (AMP)
#endif
#define NUM_PRED_MODE_CTX             2       ///< number of context models for prediction mode
#if MTK_DCM_MPM
#define NUM_ADI_CTX                   3       ///< number of context models for intra prediction
#else
#define NUM_ADI_CTX                   2
#endif
#if ADD_PLANAR_MODE && !FIXED_MPM
#define NUM_PLANARFLAG_CTX            2       ///< number of context models for planar mode flag (intra prediction)
#endif

#if DNB_INTRA_CHR_PRED_MODE
#define NUM_CHROMA_PRED_CTX           2       ///< number of context models for intra prediction (chroma)
#else
#define NUM_CHROMA_PRED_CTX           4       ///< number of context models for intra prediction (chroma)
#endif
#define NUM_INTER_DIR_CTX             4       ///< number of context models for inter prediction direction
#if MODIFIED_MVD_CODING
#define NUM_MV_RES_CTX                2       ///< number of context models for motion vector difference
#else
#define NUM_MV_RES_CTX                7       ///< number of context models for motion vector difference
#endif

#if DNB_REF_FRAME_IDX
#define NUM_REF_NO_CTX                4       ///< number of context models for reference index
#else
#define NUM_REF_NO_CTX                6       ///< number of context models for reference index
#endif
#define NUM_TRANS_SUBDIV_FLAG_CTX     10      ///< number of context models for transform subdivision flags
#define NUM_QT_CBF_CTX                5       ///< number of context models for QT CBF
#define NUM_QT_ROOT_CBF_CTX           4       ///< number of context models for QT ROOT CBF
#define NUM_DELTA_QP_CTX              4       ///< number of context models for dQP

#if UNIFIED_SCAN
#define NUM_SIG_FLAG_CTX              44      ///< number of context models for sig flag
#else
#define NUM_SIG_FLAG_CTX              50      ///< number of context models for sig flag
#endif
#if MODIFIED_LAST_CODING
#define NUM_CTX_LAST_FLAG_XY          19      ///< number of context models for PCP last flag
#else
#define NUM_CTX_LAST_FLAG_XY          26      ///< number of context models for PCP last flag
#endif
#define NUM_ONE_FLAG_CTX              30      ///< number of context models for greater than one
#define NUM_ABS_FLAG_CTX              30      ///< number of context models for magnitude

#define NUM_MVP_IDX_CTX               2       ///< number of context models for MVP index

#define NUM_ALF_FLAG_CTX              1       ///< number of context models for ALF flag
#define NUM_ALF_UVLC_CTX              2       ///< number of context models for ALF UVLC (filter length)
#define NUM_ALF_SVLC_CTX              3       ///< number of context models for ALF SVLC (filter coeff.)

#if MTK_SAO
#define NUM_AO_FLAG_CTX              1       ///< number of context models for AO flag
#define NUM_AO_UVLC_CTX              2       ///< number of context models for AO UVLC (filter length)
#define NUM_AO_SVLC_CTX              3       ///< number of context models for AO SVLC (filter coeff.)
#endif

// ====================================================================================================================
// Tables
// ====================================================================================================================

// initial probability for split flag


#if CABAC_8BITS_INIT_TABLE1

static const UChar 
INIT_SPLIT_FLAG[3][NUM_SPLIT_FLAG_CTX] = 
{
{ 122, 124, 111,},
{ 91, 123, 110,},
{ 91, 123, 110,},
};

static const UChar 
INIT_SKIP_FLAG[3][NUM_SKIP_FLAG_CTX] = 
{
{ 152, 152, 152,},
{ 195, 183, 199,},
{ 195, 183, 199,},
};

static const UChar 
INIT_MERGE_FLAG_EXT[3][NUM_MERGE_FLAG_EXT_CTX] = 
{
{ 152,},
{ 123,},
{ 108,},
};

static const UChar 
INIT_MERGE_IDX_EXT[3][NUM_MERGE_IDX_EXT_CTX] = 
{
{ 152, 152, 152, 152,},
{ 120, 121, 121, 119,},
{ 92, 137, 123, 137,},
};

static const UChar 
INIT_PART_SIZE[3][NUM_PART_SIZE_CTX] = 
{
{ 183, 152, 152, 152, 152,},
{ 152, 108, 191, 152, 152,},
{ 137, 122, 141, 78, 152,},
};

static const UChar 
INIT_PRED_MODE[3][NUM_PRED_MODE_CTX] = 
{
{ 152, 152,},
{ 152, 147,},
{ 152, 152,},
};

static const UChar 
INIT_ALF_CTRL_FLAG[3][NUM_ALF_CTRL_FLAG_CTX] = 
{
{ 152,},
{ 152,},
{ 152,},
};

static const UChar 
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX] = 
{
{ 153, 151, 108,},
{ 152, 152, 108,},
{ 137, 152, 108,},
};

static const UChar 
INIT_CHROMA_PRED_MODE[3][NUM_CHROMA_PRED_CTX] = 
{
{ 75, 137,},
{ 77, 107,},
{ 77, 122,},
};

static const UChar 
INIT_DQP[3][NUM_DELTA_QP_CTX] = 
{
{ 152, 152, 152, 152,},
{ 152, 152, 152, 152,},
{ 152, 152, 152, 152,},
};

static const UChar 
INIT_INTER_DIR[3][4] = 
{
{ 152, 152, 152, 152,},
{ 152, 152, 152, 152,},
{ 79, 63, 47, 15,},
};

static const UChar 
INIT_REF_PIC[3][NUM_REF_NO_CTX] = 
{
{ 152, 152, 152, 152,},
{ 122, 122, 137, 152,},
{ 137, 122, 152, 152,},
};

static const UChar 
INIT_MVD[3][NUM_MV_RES_CTX] = 
{
{ 152, 152,},
{ 138, 167,},
{ 138, 152,},
};

static const UChar 
INIT_QT_CBF[3][3*NUM_QT_CBF_CTX] = 
{
{ 95, 127, 152, 152, 152, 77, 92, 107, 152, 152, 77, 107, 136, 152, 152,},
{ 107, 95, 152, 152, 152, 88, 61, 136, 152, 152, 103, 75, 136, 152, 152,},
{ 107, 95, 152, 152, 152, 88, 90, 136, 152, 152, 117, 105, 136, 152, 152,},
};

static const UChar 
INIT_TRANS_SUBDIV_FLAG[3][NUM_TRANS_SUBDIV_FLAG_CTX] = 
{
{ 152, 133, 120, 120, 152, 152, 152, 152, 152, 152,},
{ 152, 122, 107, 92, 152, 152, 152, 152, 152, 152,},
{ 152, 108, 93, 92, 152, 152, 152, 152, 152, 152,},
};

static const UChar 
INIT_QT_ROOT_CBF[3][NUM_QT_ROOT_CBF_CTX] = 
{
{ 152, 152, 152, 152,},
{ 63, 79, 79, 95,},
{ 62, 79, 79, 94,},
};

static const UChar 
INIT_SIG_FLAG[3][2 * NUM_SIG_FLAG_CTX] = 
{
{ 140, 110, 109, 78, 109, 123, 123, 63, 93, 93, 93, 77, 46, 61, 47, 124, 138, 108, 76, 123, 108, 122, 75, 92, 92, 77, 104, 45, 103, 89, 149, 126, 95, 95, 60, 152, 153, 124, 125, 102, 137, 138, 138, 124, 185, 124, 91, 74, 153, 152, 135, 104, 92, 78, 152, 122, 44, 104, 136, 153, 121, 192, 192, 107, 150, 209, 120, 31, 150, 180, 151, 148, 193, 121, 123, 140, 152, 123, 104, 138, 138, 139, 125, 131, 152, 123, 78, 95,},
{ 214, 168, 123, 77, 123, 108, 123, 78, 92, 122, 123, 123, 61, 77, 137, 168, 137, 107, 46, 136, 122, 137, 91, 76, 120, 136, 151, 45, 104, 151, 137, 184, 153, 152, 163, 152, 138, 153, 154, 177, 137, 152, 167, 153, 244, 168, 137, 75, 122, 136, 195, 135, 91, 165, 123, 123, 89, 135, 152, 182, 166, 179, 193, 120, 209, 165, 194, 105, 179, 180, 166, 149, 209, 166, 152, 200, 138, 137, 149, 182, 138, 153, 124, 162, 152, 152, 108, 123,},
{ 228, 168, 123, 92, 108, 123, 138, 108, 92, 122, 153, 123, 46, 76, 151, 153, 137, 107, 61, 107, 122, 137, 121, 76, 91, 166, 122, 30, 119, 165, 137, 213, 182, 137, 163, 152, 167, 138, 154, 133, 166, 152, 167, 153, 185, 153, 122, 120, 122, 166, 166, 151, 91, 151, 167, 123, 133, 135, 166, 153, 152, 166, 194, 150, 122, 151, 166, 105, 165, 166, 152, 164, 209, 166, 152, 141, 153, 137, 151, 167, 138, 138, 124, 164, 152, 152, 152, 123,},
};

static const UChar 
INIT_LAST[3][2*NUM_CTX_LAST_FLAG_XY] = 
{
{ 152, 182, 183, 181, 166, 181, 210, 165, 194, 194, 179, 165, 194, 193, 193, 163, 178, 193, 178, 200, 214, 201, 171, 168, 198, 214, 167, 167, 181, 181, 181, 152, 152, 152, 152, 152, 152, 152,},
{ 210, 182, 227, 181, 181, 211, 152, 195, 195, 195, 195, 181, 153, 166, 195, 180, 209, 208, 208, 198, 199, 214, 183, 182, 213, 154, 167, 182, 196, 196, 166, 152, 152, 152, 152, 152, 152, 152,},
{ 166, 211, 227, 166, 181, 211, 166, 195, 195, 209, 209, 195, 167, 166, 180, 194, 194, 208, 208, 169, 184, 170, 169, 182, 198, 140, 153, 182, 211, 182, 108, 152, 152, 152, 152, 152, 152, 152,},
};


static const UChar 
INIT_ONE_FLAG[3][2*NUM_ONE_FLAG_CTX] = 
{
{ 109, 119, 120, 121, 122, 138, 136, 136, 107, 122, 139, 123, 123, 123, 123, 121, 73, 103, 118, 119, 151, 136, 136, 121, 150, 152, 137, 122, 137, 137, 138, 119, 120, 136, 122, 138, 120, 151, 136, 182, 109, 122, 123, 123, 95, 135, 26, 116, 146, 118, 166, 180, 166, 166, 166, 152, 152, 79, 181, 152,},
{ 152, 224, 208, 209, 180, 152, 136, 165, 165, 136, 153, 123, 138, 108, 152, 150, 193, 162, 147, 163, 151, 135, 150, 135, 135, 152, 137, 122, 137, 137, 152, 192, 193, 179, 165, 108, 150, 209, 179, 151, 123, 123, 137, 152, 123, 165, 177, 162, 177, 178, 151, 150, 150, 165, 136, 137, 137, 166, 137, 123,},
{ 152, 224, 208, 194, 194, 152, 121, 150, 165, 165, 124, 108, 108, 152, 182, 165, 192, 177, 147, 133, 151, 150, 150, 135, 150, 152, 137, 122, 181, 137, 152, 192, 192, 194, 194, 123, 165, 136, 136, 152, 123, 152, 152, 123, 123, 151, 162, 148, 149, 194, 137, 180, 195, 195, 166, 152, 123, 152, 152, 152,},
};

static const UChar 
INIT_ABS_FLAG[3][2*NUM_ABS_FLAG_CTX] = 
{
{ 136, 123, 138, 124, 110, 107, 137, 123, 138, 110, 123, 123, 123, 109, 125, 104, 135, 121, 122, 108, 136, 151, 122, 122, 152, 137, 137, 152, 152, 123, 121, 137, 153, 139, 110, 166, 152, 138, 124, 139, 152, 123, 123, 124, 109, 177, 164, 151, 151, 122, 151, 210, 152, 181, 152, 152, 152, 94, 181, 152,},
{ 31, 92, 108, 123, 124, 121, 107, 137, 123, 139, 108, 108, 123, 123, 139, 88, 105, 106, 107, 122, 120, 121, 136, 107, 152, 137, 137, 137, 152, 108, 134, 136, 138, 153, 154, 121, 107, 123, 123, 109, 108, 108, 108, 108, 94, 192, 164, 180, 152, 152, 180, 151, 166, 137, 152, 166, 137, 137, 137, 152,},
{ 31, 92, 137, 123, 124, 121, 107, 137, 152, 139, 137, 108, 167, 138, 139, 88, 90, 106, 107, 107, 135, 106, 107, 122, 152, 137, 122, 152, 93, 108, 119, 136, 167, 153, 139, 180, 151, 123, 109, 109, 123, 123, 123, 123, 123, 194, 195, 195, 152, 152, 166, 166, 152, 152, 152, 152, 152, 152, 152, 152,},
};

static const UChar 
INIT_MVP_IDX[3][NUM_MVP_IDX_CTX] = 
{
{ 152, 152,},
{ 137, 152,},
{ 137, 152,},
};

static const UChar 
INIT_ALF_FLAG[3][NUM_ALF_FLAG_CTX] = 
{
{ 122,},
{ 107,},
{ 151,},
};

static const UChar 
INIT_ALF_UVLC[3][NUM_ALF_UVLC_CTX] = 
{
{ 124, 167,},
{ 123, 152,},
{ 94, 123,},
};

static const UChar 
INIT_ALF_SVLC[3][NUM_ALF_SVLC_CTX] = 
{
{ 171, 152, 157,},
{ 94, 122, 186,},
{ 95, 122, 172,},
};

static const UChar 
INIT_CU_X_POS[3][NUM_CU_X_POS_CTX] = 
{
{ 152, 152,},
{ 152, 152,},
{ 152, 152,},
};

static const UChar 
INIT_CU_Y_POS[3][NUM_CU_Y_POS_CTX] = 
{
{ 152, 152,},
{ 152, 152,},
{ 152, 152,},
};

static const UChar 
INIT_AO_FLAG[3][NUM_AO_FLAG_CTX] = 
{
{ 108,},
{ 122,},
{ 78,},
};

static const UChar 
INIT_AO_UVLC[3][NUM_AO_UVLC_CTX] = 
{
{ 143, 168,},
{ 184, 138,},
{ 199, 153,},
};

static const UChar 
INIT_AO_SVLC[3][NUM_AO_SVLC_CTX] = 
{
{ 156, 152, 241,},
{ 169, 152, 167,},
{ 169, 152, 153,},
};

#else//CABAC_8BITS_INIT_TABLE1
#if CABAC_8BITS_INIT_TABLE2

static const UChar 
INIT_SPLIT_FLAG[3][NUM_SPLIT_FLAG_CTX] = 
{
{ 137, 124, 95,},
{ 90, 123, 109,},
{ 60, 123, 109,},
};

static const UChar 
INIT_SKIP_FLAG[3][NUM_SKIP_FLAG_CTX] = 
{
{ 107, 107, 107,},
{ 226, 214, 216,},
{ 226, 214, 200,},
};

static const UChar 
INIT_MERGE_FLAG_EXT[3][NUM_MERGE_FLAG_EXT_CTX] = 
{
{ 107,},
{ 108,},
{ 107,},
};

static const UChar 
INIT_MERGE_IDX_EXT[3][NUM_MERGE_IDX_EXT_CTX] = 
{
{ 107, 107, 107, 107,},
{ 105, 151, 151, 134,},
{ 91, 122, 153, 122,},
};

static const UChar 
INIT_PART_SIZE[3][NUM_PART_SIZE_CTX] = 
{
{ 169, 107, 107, 107, 107,},
{ 137, 107, 207, 107, 107,},
{ 137, 137, 141, 107, 107,},
};

static const UChar 
INIT_PRED_MODE[3][NUM_PRED_MODE_CTX] = 
{
{ 107, 107,},
{ 107, 178,},
{ 107, 107,},
};

static const UChar 
INIT_ALF_CTRL_FLAG[3][NUM_ALF_CTRL_FLAG_CTX] = 
{
{ 107,},
{ 107,},
{ 107,},
};

static const UChar 
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX] = 
{
{ 184, 106, 107,},
{ 122, 122, 107,},
{ 137, 122, 107,},
};

static const UChar 
INIT_CHROMA_PRED_MODE[3][NUM_CHROMA_PRED_CTX] = 
{
{ 13, 122,},
{ 91, 137,},
{ 76, 122,},
};

static const UChar 
INIT_DQP[3][NUM_DELTA_QP_CTX] = 
{
{ 107, 107, 107, 107,},
{ 107, 107, 107, 107,},
{ 107, 107, 107, 107,},
};

static const UChar 
INIT_INTER_DIR[3][4] = 
{
{ 107, 107, 107, 107,},
{ 107, 107, 107, 107,},
{ 47, 31, 15, 12,},
};

static const UChar 
INIT_REF_PIC[3][NUM_REF_NO_CTX] = 
{
{ 107, 107, 107, 107,},
{ 137, 122, 137, 107,},
{ 137, 92, 122, 107,},
};

static const UChar 
INIT_MVD[3][NUM_MV_RES_CTX] = 
{
{ 107, 107,},
{ 138, 183,},
{ 153, 183,},
};

static const UChar 
INIT_QT_CBF[3][3*NUM_QT_CBF_CTX] = 
{
{ 79, 111, 107, 107, 107, 61, 76, 106, 107, 107, 76, 76, 121, 107, 107,},
{ 107, 79, 107, 107, 107, 72, 29, 121, 107, 107, 102, 44, 121, 107, 107,},
{ 92, 79, 107, 107, 107, 87, 59, 121, 107, 107, 117, 74, 121, 107, 107,},
};

static const UChar 
INIT_TRANS_SUBDIV_FLAG[3][NUM_TRANS_SUBDIV_FLAG_CTX] = 
{
{ 107, 148, 120, 105, 107, 107, 107, 107, 107, 107,},
{ 107, 107, 92, 91, 107, 107, 107, 107, 107, 107,},
{ 107, 107, 92, 91, 107, 107, 107, 107, 107, 107,},
};

static const UChar 
INIT_QT_ROOT_CBF[3][NUM_QT_ROOT_CBF_CTX] = 
{
{ 107, 107, 107, 107,},
{ 31, 63, 63, 79,},
{ 15, 47, 63, 63,},
};

static const UChar 
INIT_SIG_FLAG[3][2 * NUM_SIG_FLAG_CTX] = 
{
{ 140, 109, 93, 46, 78, 108, 108, 31, 62, 62, 62, 31, 14, 14, 15, 124, 138, 107, 29, 153, 107, 122, 43, 76, 91, 106, 88, 43, 72, 103, 134, 111, 79, 79, 43, 168, 123, 124, 140, 71, 137, 153, 123, 139, 201, 139, 75, 57, 123, 122, 105, 73, 76, 62, 153, 122, 12, 58, 106, 138, 136, 224, 224, 91, 166, 241, 135, 14, 135, 227, 166, 149, 225, 136, 108, 125, 107, 123, 103, 153, 153, 154, 125, 101, 183, 153, 62, 109,},
{ 246, 169, 138, 46, 138, 107, 153, 46, 61, 107, 138, 108, 14, 60, 122, 168, 122, 106, 14, 121, 152, 152, 75, 59, 105, 121, 121, 43, 58, 151, 137, 215, 168, 137, 164, 168, 153, 138, 139, 163, 152, 137, 168, 138, 247, 199, 152, 74, 122, 151, 212, 150, 75, 166, 123, 138, 88, 135, 152, 198, 197, 180, 225, 135, 211, 196, 226, 104, 195, 166, 167, 119, 226, 182, 167, 246, 168, 152, 149, 183, 153, 153, 124, 208, 167, 167, 122, 123,},
{ 230, 184, 153, 61, 92, 153, 153, 92, 76, 122, 153, 108, 29, 75, 167, 153, 122, 91, 14, 106, 152, 167, 106, 44, 105, 182, 137, 12, 134, 151, 152, 230, 168, 152, 149, 152, 168, 138, 154, 163, 167, 183, 168, 184, 171, 168, 122, 105, 107, 167, 197, 121, 60, 136, 198, 108, 133, 135, 167, 168, 167, 197, 196, 135, 137, 166, 182, 119, 150, 167, 167, 195, 226, 197, 167, 171, 139, 137, 166, 168, 153, 123, 124, 195, 168, 153, 153, 108,},
};

static const UChar 
INIT_LAST[3][2*NUM_CTX_LAST_FLAG_XY] = 
{
{ 152, 213, 184, 182, 167, 167, 227, 211, 226, 180, 210, 211, 225, 240, 224, 194, 194, 224, 224, 216, 231, 232, 202, 214, 184, 201, 198, 213, 182, 167, 182, 107, 107, 107, 107, 107, 107, 107,},
{ 212, 198, 229, 182, 212, 243, 152, 212, 242, 211, 196, 197, 153, 167, 196, 211, 226, 210, 240, 215, 230, 246, 184, 198, 245, 154, 168, 213, 198, 213, 197, 107, 107, 107, 107, 107, 107, 107,},
{ 212, 228, 229, 197, 197, 243, 167, 226, 241, 226, 226, 242, 153, 182, 226, 226, 225, 240, 240, 215, 215, 201, 170, 183, 230, 110, 154, 183, 243, 213, 107, 107, 107, 107, 107, 107, 107, 107,},
};

static const UChar 
INIT_ONE_FLAG[3][2*NUM_ONE_FLAG_CTX] = 
{
{ 108, 119, 120, 136, 152, 138, 121, 121, 106, 137, 124, 138, 138, 153, 153, 151, 72, 72, 118, 134, 106, 136, 136, 151, 151, 168, 122, 122, 137, 137, 123, 134, 135, 151, 152, 138, 105, 136, 136, 198, 108, 107, 138, 47, 79, 135, 39, 116, 162, 118, 152, 226, 152, 197, 182, 153, 167, 78, 167, 183,},
{ 168, 241, 240, 241, 211, 153, 151, 166, 166, 151, 154, 122, 153, 107, 183, 135, 224, 178, 163, 193, 167, 150, 165, 165, 135, 153, 137, 137, 152, 152, 168, 224, 224, 210, 196, 107, 166, 211, 180, 166, 108, 108, 122, 152, 123, 196, 208, 178, 178, 149, 167, 166, 135, 181, 166, 122, 167, 167, 167, 167,},
{ 137, 241, 240, 225, 211, 122, 151, 166, 181, 166, 139, 107, 92, 107, 183, 181, 209, 178, 147, 133, 167, 165, 165, 135, 150, 168, 137, 152, 167, 152, 152, 224, 224, 241, 210, 123, 196, 136, 151, 137, 108, 167, 167, 108, 123, 181, 178, 193, 195, 226, 152, 196, 226, 226, 167, 122, 123, 107, 107, 107,},
};

static const UChar 
INIT_ABS_FLAG[3][2*NUM_ABS_FLAG_CTX] = 
{
{ 121, 153, 123, 139, 79, 91, 122, 138, 123, 94, 138, 138, 123, 108, 125, 58, 120, 121, 122, 107, 136, 106, 152, 137, 122, 122, 122, 122, 107, 138, 121, 122, 123, 139, 109, 182, 108, 138, 154, 124, 137, 108, 123, 109, 78, 193, 195, 167, 152, 107, 181, 212, 152, 182, 197, 137, 183, 93, 182, 167,},
{ 44, 91, 107, 138, 124, 106, 106, 137, 153, 154, 107, 107, 153, 138, 139, 72, 89, 90, 91, 107, 105, 136, 121, 152, 122, 137, 137, 107, 168, 92, 119, 136, 153, 108, 124, 90, 106, 153, 108, 109, 137, 137, 92, 108, 93, 148, 165, 211, 167, 167, 211, 166, 197, 167, 167, 167, 152, 152, 152, 122,},
{ 13, 106, 122, 138, 139, 136, 106, 122, 107, 139, 107, 107, 153, 138, 139, 72, 59, 75, 91, 137, 105, 75, 106, 152, 122, 152, 122, 122, 107, 107, 104, 151, 183, 123, 154, 196, 151, 123, 108, 93, 108, 108, 108, 108, 108, 210, 226, 227, 152, 152, 197, 167, 167, 167, 152, 152, 152, 107, 107, 107,},
};

static const UChar 
INIT_MVP_IDX[3][NUM_MVP_IDX_CTX] = 
{
{ 107, 107,},
{ 152, 107,},
{ 152, 107,},
};

static const UChar 
INIT_ALF_FLAG[3][NUM_ALF_FLAG_CTX] = 
{
{ 137,},
{ 106,},
{ 121,},
};

static const UChar 
INIT_ALF_UVLC[3][NUM_ALF_UVLC_CTX] = 
{
{ 154, 168,},
{ 123, 168,},
{ 93, 153,},
};

static const UChar 
INIT_ALF_SVLC[3][NUM_ALF_SVLC_CTX] = 
{
{ 187, 122, 142,},
{ 63, 107, 187,},
{ 63, 122, 142,},
};

static const UChar 
INIT_CU_X_POS[3][NUM_CU_X_POS_CTX] = 
{
{ 107, 107,},
{ 168, 122,},
{ 137, 122,},
};

static const UChar 
INIT_CU_Y_POS[3][NUM_CU_Y_POS_CTX] = 
{
{ 107, 107,},
{ 168, 107,},
{ 137, 122,},
};

static const UChar 
INIT_AO_FLAG[3][NUM_AO_FLAG_CTX] = 
{
{ 107,},
{ 107,},
{ 92,},
};

static const UChar 
INIT_AO_UVLC[3][NUM_AO_UVLC_CTX] = 
{
{ 143, 199,},
{ 155, 153,},
{ 231, 138,},
};

static const UChar 
INIT_AO_SVLC[3][NUM_AO_SVLC_CTX] = 
{
{ 172, 107, 244,},
{ 185, 153, 183,},
{ 200, 153, 168,},
};


#else // CABAC_8BITS_INIT_TABLE2



static const Short
INIT_SPLIT_FLAG[3][NUM_SPLIT_FLAG_CTX][2] =
{
  {
    {   -7,   68 }, {  -10,   87 }, {  -10,  105 }
  },
  {
    {  -14,   71 }, {   -6,   73 }, {   -6,   91 }
  },
  {
    {  -14,   71 }, {   -7,   74 }, {  -10,   92 }
  }
};

// initial probability for skip flag
static const Short
INIT_SKIP_FLAG[3][NUM_SKIP_FLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

// initial probability for skip flag
#if DNB_ALF_CTRL_FLAG
static const Short
INIT_ALF_CTRL_FLAG[3][NUM_ALF_CTRL_FLAG_CTX][2] =
{
  {
    {    0,   64 }
  },
  {
    {    0,   64 }
  },
  {
    {    0,   64 }
  }
};
#else
static const Short
INIT_ALF_CTRL_FLAG[3][NUM_ALF_CTRL_FLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#endif

// initial probability for merge flag
#if DNB_MERGE_FLAG
static const Short
INIT_MERGE_FLAG_EXT[3][NUM_MERGE_FLAG_EXT_CTX][2] =
{
  {
    {    0,   64 }
  },
  {
    {    0,   64 }
  },
  {
    {    0,   64 }
  }
};
#else
static const Short
INIT_MERGE_FLAG_EXT[3][NUM_MERGE_FLAG_EXT_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#endif

static const Short
INIT_MERGE_IDX_EXT[3][NUM_MERGE_IDX_EXT_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    1,   65 }, {    6,   42 }, {   -7,   75 }, {   -4,   72 }
  }
};

// initial probability for PU size
static const Short
INIT_PART_SIZE[3][NUM_PART_SIZE_CTX][2] =
{
  {
    {    0,   73 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }
  },
  {
    {   -1,   64 }, {   -3,   63 }, {    6,   78 }, {    0,   64 }, 
    {    0,   64 }
  },
  {
    {    6,   50 }, {   -1,   56 }, {   13,   53 }, {  -11,   76 }, 
    {  -11,   70 }
  }
};

#if AMP
// initial probability for AMP split position (X)
static const Short
INIT_CU_X_POS[3][NUM_CU_X_POS_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {   -1,   59 }, {    0,   63 }
  },
  {
    {   -1,   55 }, {   -3,   67 }
  }
};

// initial probability for AMP split position (Y)
static const Short
INIT_CU_Y_POS[3][NUM_CU_Y_POS_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   57 }, {   -2,   66 }
  },
  {
    {   -3,   61 }, {   -3,   66 }
  }
};
#endif

// initial probability for prediction mode
static const Short
INIT_PRED_MODE[3][NUM_PRED_MODE_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {  -25,   89 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};

// initial probability for intra direction of luma
#if MTK_DCM_MPM
static const Short
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX][2] =
{
  {
    {    2,   54 }, {  -3,   65  }, {   -3,   65 }
  },
  {
    {    0,   50 }, {  -2,   61  }, {   -2,   61 }
  },
  {
    {    0,   51 }, {  1,   55   }, {    1,   55 }
  }
};
#else
static const Short
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX][2] =
{
  {
    {    2,   54 }, {   -3,   65 }
  },
  {
    {    0,   50 }, {   -2,   61 }
  },
  {
    {    0,   51 }, {    1,   55 }
  }
};
#endif

#if ADD_PLANAR_MODE && !FIXED_MPM
// initial probability for planar mode flag
static const Short
INIT_PLANARFLAG[3][NUM_PLANARFLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};
#endif

// initial probability for intra direction of chroma
#if DNB_INTRA_CHR_PRED_MODE
static const Short
INIT_CHROMA_PRED_MODE[3][NUM_CHROMA_PRED_CTX][2] =
{
  {
    {  0,   64 }, {   0,   64 }
  },
  {
    {  0,   64 }, {   0,   64 }
  },
  {
    {  0,   64 }, {   0,   64 }
  }
};
#else
static const Short
INIT_CHROMA_PRED_MODE[3][4][2] =
{
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  },
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  },
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  }
};
#endif

// initial probability for temporal direction
static const Short
INIT_INTER_DIR[3][4][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {   -2,   58 }, {   -5,   70 }, {   -9,   85 }, {    1,   61 }
  }
};

// initial probability for motion vector difference
#if MODIFIED_MVD_CODING
static const
Short INIT_MVD[3][NUM_MV_RES_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};
#else
static const Short INIT_MVD[3][14][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }
  },
  {
    {   -6,   80 }, {   -6,   84 }, {   -9,   90 }, {    4,   62 }, 
    {   13,   55 }, {    2,   70 }, {    8,   74 }, {   -6,   77 }, 
    {   -7,   84 }, {   -9,   89 }, {    5,   59 }, {   10,   62 }, 
    {    4,   68 }, {    7,   75 }
  },
  {
    {   -4,   75 }, {   -5,   82 }, {  -12,   94 }, {    7,   55 }, 
    {   11,   59 }, {    6,   63 }, {    8,   71 }, {   -2,   71 }, 
    {   -5,   81 }, {  -21,  111 }, {    6,   58 }, {   10,   60 }, 
    {    5,   64 }, {   10,   67 }
  }
};
#endif

// initial probability for reference frame index
#if DNB_REF_FRAME_IDX
static const Short
INIT_REF_PIC[3][NUM_REF_NO_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {   -6,   59 }, {  -17,   96 }, {    1,   59 }, {    0,   64 }
  },
  {
    {   -9,   55 }, {  -12,   86 }, {  -18,   55 }, {    0,   64 }
  }
};
#else
static const Short
INIT_REF_PIC[3][NUM_REF_NO_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }
  },
  {
    {   -6,   59 }, {  -10,   75 }, {   -8,   75 }, {  -17,   96 }, 
    {    1,   59 }, {    0,   64 }
  },
  {
    {   -9,   55 }, {   -9,   71 }, {   -9,   76 }, {  -12,   86 }, 
    {  -18,   55 }, {    0,   64 }
  }
};
#endif

// initial probability for dQP
static const Short
INIT_DQP[3][NUM_DELTA_QP_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

static const Short
INIT_QT_CBF[3][3*NUM_QT_CBF_CTX][2] =
{
  {
    {  -22,  116 }, {   -5,   75 }, {  -16,  112 }, {  -16,  111 }, {  -32,  165 },
    {  -35,  116 }, {  -12,   61 }, {   -9,   73 }, {  -10,   75 }, {  -14,   96 }, 
    {  -29,  104 }, {  -12,   59 }, {   -5,   65 }, {   -6,   67 }, {  -11,   90 }
  },
  {
    {  -18,   98 }, {  -41,  120 }, {  -29,  117 }, {  -23,  108 }, {  -35,  143 },
    {  -46,  114 }, {  -42,  119 }, {  -11,   74 }, {  -19,   90 }, {  -42,  139 }, 
    {  -43,  107 }, {  -41,  118 }, {  -17,   86 }, {  -25,  101 }, {  -14,   91 }
  },
  {
    {  -11,   80 }, {  -32,   83 }, {  -19,   89 }, {  -16,   85 }, {  -19,  102 },
    {  -22,   52 }, {  -48,  123 }, {   -7,   68 }, {  -37,  121 }, {  -58,  164 }, 
    {  -19,   45 }, {  -48,  123 }, {  -21,   94 }, {   -9,   73 }, {  -42,  138 }
  }
};

static const Short
INIT_QT_ROOT_CBF[3][NUM_QT_ROOT_CBF_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
  },
  {
    {  -22,   85 }, {  -15,   86 }, {  -13,   84 }, {  -23,  116 },
  },
  {
    {  -36,  103 }, {  -21,   95 }, {  -21,   97 }, {  -24,  114 },
  }
};

#if MODIFIED_LAST_CODING
static const Short
INIT_LAST[3][2*NUM_CTX_LAST_FLAG_XY][2] =
{
  {
    {   14,   25 }, {   10,   38 }, {   12,   41 },
    {   22,   25 }, {   10,   37 }, {    9,   40 },
    {   17,   19 }, {   15,   21 }, {   16,   18 }, {   17,   16 }, {   18,   13 }, {   16,   18 },
    {   23,   -2 }, {   20,   -2 }, {   24,   -9 }, {   24,   -7 }, {   18,   10 }, {    9,   24 }, {   22,   -7 },
    {   37,    9 }, {   35,   14 }, {   53,   -5 },
    {   31,   21 }, {   13,   39 }, {   21,   28 },
    {   35,    2 }, {   22,   17 }, {   18,   23 }, {    9,   40 }, {   14,   32 }, {    7,   44 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {   12,   35 }, {   12,   40 }, {   13,   47 },
    {   21,   16 }, {   24,   10 }, {   23,   15 },
    {   13,   34 }, {   22,   10 }, {   29,   10 }, {   29,   -8 }, {   27,   -5 }, {   20,    7 },
    {   25,   13 }, {   30,   -8 }, {   35,  -20 }, {   37,  -29 }, {   42,  -41 }, {   42,  -45 }, {   43,  -46 },

    {   21,   36 }, {   22,   37 }, {   35,   27 },
    {   11,   53 }, {   22,   27 }, {   27,   18 },
    {   -3,   83 }, {   14,   35 }, {   23,   18 }, {   27,    3 }, {   32,  -11 }, {   17,   11 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {   12,   35 }, {   12,   40 }, {   13,   47 },
    {   21,   16 }, {   24,   10 }, {   23,   15 },
    {   13,   34 }, {   22,   10 }, {   29,   10 }, {   29,   -8 }, {   27,   -5 }, {   20,    7 },
    {   25,   13 }, {   30,   -8 }, {   35,  -20 }, {   37,  -29 }, {   42,  -41 }, {   42,  -45 }, {   43,  -46 },

    {   21,   36 }, {   22,   37 }, {   35,   27 },
    {   11,   53 }, {   22,   27 }, {   27,   18 },
    {   -3,   83 }, {   14,   35 }, {   23,   18 }, {   27,    3 }, {   32,  -11 }, {   17,   11 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#else
static const Short
INIT_LAST_X[3][2*NUM_CTX_LAST_FLAG_XY][2] =
{
  {
    {   8,  31 }, {   8,  39 }, {   8,  48 }, {  12,  31 },
    {   8,  38 }, {   6,  45 }, {   3,  46 }, {  -1,  56 },
    {  18,  16 }, {  14,  22 }, {  15,  22 }, {  16,  17 },
    {  16,  16 }, {  12,  24 }, {  -4,  59 }, {  33, -26 },
    {  18,   1 }, {  20,   2 }, {  22,  -1 }, {  17,  14 },
    {  11,  21 }, {  31, -24 }, {  38, -38 }, {  12,  11 },
    {  -4,  47 }, { -13,  69 }, {  32,  11 }, {  25,  27 },
    {  50,  -1 }, {  32,  20 }, {  12,  38 }, {  18,  32 },
    {  12,  41 }, {   5,  70 }, {  40,  -6 }, {  14,  29 },
    {  17,  26 }, {   7,  43 }, {  15,  26 }, {  15,  27 },
    {   9,  51 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {   9,  40 }, {   9,  44 }, {  10,  52 }, {  17,  24 },
    {  21,  15 }, {  20,  20 }, {  14,  29 }, {   8,  46 },
    {   7,  46 }, {  18,  18 }, {  26,   3 }, {  26,   0 },
    {  25,   2 }, {  15,  18 }, {   9,  27 }, {  23,  20 },
    {  19,  16 }, {  27,  -1 }, {  33, -19 }, {  38, -30 },
    {  41, -39 }, {  40, -36 }, {  26, -12 }, {   3,  20 },
    {  -7,  33 }, {  -4,  20 }, {  16,  42 }, {  16,  45 },
    {  49,   7 }, {   6,  61 }, {  17,  36 }, {  24,  24 },
    {  24,  21 }, {  61, -22 }, { -11, 101 }, {   6,  53 },
    {  14,  40 }, {  17,  30 }, {  20,  22 }, { -10,  67 },
    {  -6,  68 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {   9,  40 }, {   9,  44 }, {  10,  52 }, {  17,  24 },
    {  21,  15 }, {  20,  20 }, {  14,  29 }, {   8,  46 },
    {   7,  46 }, {  18,  18 }, {  26,   3 }, {  26,   0 },
    {  25,   2 }, {  15,  18 }, {   9,  27 }, {  23,  20 },
    {  19,  16 }, {  27,  -1 }, {  33, -19 }, {  38, -30 },
    {  41, -39 }, {  40, -36 }, {  26, -12 }, {   3,  20 },
    {  -7,  33 }, {  -4,  20 }, {  16,  42 }, {  16,  45 },
    {  49,   7 }, {   6,  61 }, {  17,  36 }, {  24,  24 },
    {  24,  21 }, {  61, -22 }, { -11, 101 }, {   6,  53 },
    {  14,  40 }, {  17,  30 }, {  20,  22 }, { -10,  67 },
    {  -6,  68 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  }
};

static const Short
INIT_LAST_Y[3][2*NUM_CTX_LAST_FLAG_XY][2] =
{
  {
    {  19,  19 }, {  12,  36 }, {  16,  34 }, {  22,  18 },
    {  12,  35 }, {  12,  35 }, {  12,  32 }, {   5,  46 },
    {  16,  21 }, {  15,  20 }, {  17,  13 }, {  17,  14 },
    {  19,  10 }, {  19,  12 }, {   4,  37 }, {  13,  22 },
    {  22,  -4 }, {  27, -19 }, {  26, -12 }, {  18,   6 },
    {   6,  27 }, {  12,  10 }, {  34, -33 }, {  38, -42 },
    {  24, -15 }, {  14,   7 }, {  41,   7 }, {  45,   1 },
    {  56,  -9 }, {  30,  22 }, {  14,  40 }, {  23,  24 },
    {  19,  32 }, {  25,  26 }, {  29,  10 }, {  29,   4 },
    {  19,  20 }, {  10,  37 }, {  12,  38 }, {  -2,  60 },
    { -37, 114 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {  15,  29 }, {  14,  36 }, {  15,  41 }, {  25,   8 },
    {  26,   4 }, {  25,  10 }, {  21,  16 }, {  12,  34 },
    {  18,  21 }, {  25,   1 }, {  32, -12 }, {  32, -15 },
    {  29, -11 }, {  24,  -5 }, {  13,  12 }, {  27,   6 },
    {  40, -32 }, {  42, -39 }, {  40, -39 }, {  45, -51 },
    {  43, -51 }, {  45, -56 }, {  38, -46 }, {  15,  -6 },
    {  11,  -4 }, {  11,  -4 }, {  26,  29 }, {  28,  29 },
    {  21,  46 }, {  15,  45 }, {  27,  17 }, {  29,  12 },
    {  23,  15 }, {  -1,  57 }, {   6,  65 }, {  22,  17 },
    {  31,  -4 }, {  37, -25 }, {  43, -44 }, {  43, -46 },
    {  48, -44 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {  15,  29 }, {  14,  36 }, {  15,  41 }, {  25,   8 },
    {  26,   4 }, {  25,  10 }, {  21,  16 }, {  12,  34 },
    {  18,  21 }, {  25,   1 }, {  32, -12 }, {  32, -15 },
    {  29, -11 }, {  24,  -5 }, {  13,  12 }, {  27,   6 },
    {  40, -32 }, {  42, -39 }, {  40, -39 }, {  45, -51 },
    {  43, -51 }, {  45, -56 }, {  38, -46 }, {  15,  -6 },
    {  11,  -4 }, {  11,  -4 }, {  26,  29 }, {  28,  29 },
    {  21,  46 }, {  15,  45 }, {  27,  17 }, {  29,  12 },
    {  23,  15 }, {  -1,  57 }, {   6,  65 }, {  22,  17 },
    {  31,  -4 }, {  37, -25 }, {  43, -44 }, {  43, -46 },
    {  48, -44 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  }
};
#endif

#if UNIFIED_SCAN
static const Short
INIT_SIG_FLAG[3][2 * NUM_SIG_FLAG_CTX][2] =
{
  {
    // Luma 4x4
    { -11, 103 }, { -10, 94 }, { -9, 87 }, { -14, 89 }, 
    { -11, 93 }, { -11, 92 }, { -11, 91 }, { -17, 97 }, 
    { -13, 91 }, { -12, 89 }, { -10, 89 }, { -19, 101 }, 
    { -19, 94 }, { -16, 93 }, { -14, 93 },
    // Luma 8x8
    { -8, 90 }, { -4, 72 }, { 0, 58 }, { -6, 54 }, 
    { -8, 80 }, { -4, 70 }, { 1, 57 }, { -4, 54 }, 
    { -8, 72 }, { -3, 63 }, { 3, 54 }, { -3, 55 }, 
    { -17, 79 }, { -5, 59 }, { -1, 56 }, { -1, 53 }, 
    // Luma 16x16
    { -15, 119 }, { -14, 104 }, { -15, 106 }, { -4, 49 }, 
    { 0, 62 }, { -2, 72 }, { -7, 88 }, { -15, 112 }, 
    { -4, 28 }, { 1, 54 }, { -4, 72 }, { -7, 82 }, 
    { -10, 96 },
    // Chroma 4x4
    { 34, 22 }, { 19, 41 }, { -11, 83 }, { -58, 144 }, 
    { 6, 58 }, { 8, 52 }, { 1, 64 }, { -59, 149 }, 
    { -14, 83 }, { 19, 30 }, { 60, -35 }, { 1, 56 }, 
    { -50, 124 }, { -53, 137 }, { 7, 47 },
    // Chroma 8x8
    { 14, 48 }, { 6, 50 }, { -30, 103 }, { -35, 74 }, 
    { -7, 68 }, { 14, 36 }, { 8, 43 }, { -25, 63 }, 
    { -37, 104 }, { -21, 83 }, { 9, 43 }, { 7, 22 }, 
    { -37, 80 }, { 2, 24 }, { -36, 103 }, { 1, 57 }, 
    // Chroma 16x16
    { 15, 59 }, { 7, 56 }, { 5, 57 }, { 14, 11 }, 
    { 10, 45 }, { 7, 53 }, { 5, 61 }, { 11, 59 }, 
    { -9, 38 }, { 5, 46 }, { 7, 49 }, { 10, 48 }, 
    { 13, 47 } 
  },
  {
    // Luma 4x4
    { -2, 74 }, { -5, 76 }, { -6, 72 }, { -19, 89 }, 
    { -8, 77 }, { -8, 75 }, { -9, 76 }, { -17, 88 }, 
    { -10, 73 }, { -10, 75 }, { -4, 69 }, { -22, 100 }, 
    { -20, 88 }, { -19, 89 }, { -9, 75 },
    // Luma 8x8
    { 2, 61 }, { -1, 56 }, { -2, 51 }, { -15, 67 }, 
    { -9, 66 }, { -5, 58 }, { -1, 49 }, { -8, 55 }, 
    { -16, 72 }, { -10, 62 }, { -1, 49 }, { -4, 52 }, 
    { -24, 88 }, { -20, 84 }, { -10, 70 }, { -3, 59 }, 
    // Luma 16x16
    { 0, 78 }, { 0, 66 }, { -3, 68 }, { 2, 31 }, 
    { 3, 53 }, { 0, 65 }, { -3, 74 }, { -9, 93 }, 
    { -4, 20 }, { 4, 44 }, { 1, 57 }, { 0, 65 }, 
    { -1, 72 }, 
    // Chroma 4x4
    { 41, 3 }, { 17, 37 }, { 16, 33 }, { -27, 89 }, 
    { 0, 57 }, { 13, 34 }, { 75, -70 }, { 4, 43 }, 
    { -24, 84 }, { -11, 68 }, { 6, 52 }, { 20, 40 }, 
    { -29, 79 }, { -50, 124 }, { 12, 38 },
    // Chroma 8x8
    { 18, 31 }, { 22, 11 }, { 13, 10 }, { -32, 58 }, 
    { 3, 36 }, { 25, -6 }, { 21, -3 }, { 37, -34 }, 
    { -30, 81 }, { 36, -36 }, { 27, -2 }, { 11, 36 }, 
    { -56, 143 }, { 20, 3 }, { 16, 19 }, { 19, 46 }, 
    // Chroma 16x16
    { 28, 29 }, { 16, 35 }, { 11, 39 }, { 26, -18 }, 
    { 10, 44 }, { 4, 58 }, { -2, 71 }, { -11, 94 }, 
    { 0, 0 }, { 5, 45 }, { 5, 49 }, { 9, 45 }, 
    { 20, 32 }
  },
  {
    // Luma 4x4
    { -2, 74 }, { -5, 76 }, { -6, 72 }, { -19, 89 }, 
    { -8, 77 }, { -8, 75 }, { -9, 76 }, { -17, 88 }, 
    { -10, 73 }, { -10, 75 }, { -4, 69 }, { -22, 100 }, 
    { -20, 88 }, { -19, 89 }, { -9, 75 },
    // Luma 8x8
    { 2, 61 }, { -1, 56 }, { -2, 51 }, { -15, 67 }, 
    { -9, 66 }, { -5, 58 }, { -1, 49 }, { -8, 55 }, 
    { -16, 72 }, { -10, 62 }, { -1, 49 }, { -4, 52 }, 
    { -24, 88 }, { -20, 84 }, { -10, 70 }, { -3, 59 }, 
    // Luma 16x16
    { 0, 78 }, { 0, 66 }, { -3, 68 }, { 2, 31 }, 
    { 3, 53 }, { 0, 65 }, { -3, 74 }, { -9, 93 }, 
    { -4, 20 }, { 4, 44 }, { 1, 57 }, { 0, 65 }, 
    { -1, 72 }, 
    // Chroma 4x4
    { 41, 3 }, { 17, 37 }, { 16, 33 }, { -27, 89 }, 
    { 0, 57 }, { 13, 34 }, { 75, -70 }, { 4, 43 }, 
    { -24, 84 }, { -11, 68 }, { 6, 52 }, { 20, 40 }, 
    { -29, 79 }, { -50, 124 }, { 12, 38 },
    // Chroma 8x8
    { 18, 31 }, { 22, 11 }, { 13, 10 }, { -32, 58 }, 
    { 3, 36 }, { 25, -6 }, { 21, -3 }, { 37, -34 }, 
    { -30, 81 }, { 36, -36 }, { 27, -2 }, { 11, 36 }, 
    { -56, 143 }, { 20, 3 }, { 16, 19 }, { 19, 46 }, 
    // Chroma 16x16
    { 28, 29 }, { 16, 35 }, { 11, 39 }, { 26, -18 }, 
    { 10, 44 }, { 4, 58 }, { -2, 71 }, { -11, 94 }, 
    { 0, 0 }, { 5, 45 }, { 5, 49 }, { 9, 45 }, 
    { 20, 32 }
  }
};
#else
static const Short
INIT_SIG_FLAG[3][2 * NUM_SIG_FLAG_CTX][2] =
{
  {
    // Luma 4x4
    {   -4,   94 }, {   -2,   84 }, {   -4,   83 }, {   16,   39 },
    {    0,   83 }, {   -1,   81 }, {   -4,   82 }, {  -16,   91 },
    {   -9,   86 }, {   -8,   89 }, {   -2,   82 }, {  -14,   92 },
    {  -16,   91 }, {   13,   44 }, {  -12,   91 },
    // Luma 8x8
    {  -10,   91 }, {   -9,   76 }, {   -3,   61 }, {    1,   46 },
    {  -13,   84 }, {  -13,   81 }, {   -2,   60 }, {   -5,   61 },
    {  -11,   77 }, {  -10,   76 }, {   -3,   65 }, {   -3,   63 },
    {   -4,   59 }, {  -14,   79 }, {    1,   61 }, {  -33,  126 },
    // Luma 16x16
    {   -5,   99 }, {   -9,   92 }, {   -5,   90 }, {   -7,   83 },
    {   -3,   37 }, {   -1,   62 }, {   -5,   81 }, {   -2,   41 },
    {    1,   64 }, {   -6,   82 }, {    0,   12 }, {    1,   43 },
    {    0,   57 }, {   -1,   66 }, {   -4,   79 },
    // Luma 32x32
    {   -3,  102 }, {  -17,  114 }, {   -7,   97 }, {  -12,   96 },
    // Chroma 4x4
    {   29,   42 }, {    1,   74 }, {   -6,   77 }, {   23,   41 },
    {   13,   63 }, {   -6,   88 }, {   -8,   80 }, {   35,   17 },
    {    0,   70 }, {  -17,  100 }, {  -25,  111 }, {  -39,  122 },
    {   26,   29 }, {  -36,  114 }, {  -41,  130 },
    // Chroma 8x8
    {   19,   45 }, {    8,   48 }, {    2,   40 }, {   -3,   40 },
    {    8,   46 }, {   16,   31 }, {  -10,   73 }, {  -51,  114 },
    {    2,   52 }, {   -9,   71 }, {  -37,  118 }, {   -8,   38 },
    {  -10,   64 }, {  -47,  113 }, { -105,  234 }, {  -32,  123 },
    // Chroma 16x16
    {   23,   51 }, {    0,   69 }, {   10,   54 }, {    0,   65 },
    {   -1,   42 }, {    4,   51 }, {   -3,   70 }, {  -12,   61 },
    {   -4,   50 }, {   -2,   68 }, {   -18,  57 }, {    5,   38 },
    {    1,   54 }, {   -3,   66 }, {   -9,   80 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    // Luma 4x4
    {    6,   67 }, {   -1,   72 }, {   -9,   84 }, {  -29,  100 },
    {    4,   65 }, {    0,   70 }, {   -7,   80 }, {  -34,  117 },
    {  -11,   79 }, {  -14,   91 }, {  -15,   93 }, {  -34,  119 },
    {   26,   26 }, {  -34,  111 }, {  -35,  126 },
    // Luma 8x8
    {    2,   69 }, {  -10,   82 }, {   -1,   57 }, {   -7,   59 },
    {   -5,   71 }, {  -13,   85 }, {  -28,  107 }, {  -29,  100 },
    {  -21,   93 }, {  -28,  108 }, {   -6,   71 }, {  -48,  136 },
    {  -35,  111 }, {  -10,   74 }, {    4,   56 }, {    5,   56 },
    // Luma 16x16
    {    0,   75 }, {    1,   59 }, {   -1,   67 }, {   -3,   67 },
    {    6,   29 }, {    2,   57 }, {   -5,   77 }, {   -5,   44 },
    {   -4,   64 }, {   -7,   78 }, {   -9,   34 }, {    2,   43 },
    {    2,   53 }, {    2,   60 }, {   -8,   83 },
    // Luma 32x32
    {   -7,   88 }, {    5,   52 }, {   -4,   74 }, {    5,   56 },
    // Chroma 4x4
    {  -15,  103 }, {   25,   19 }, {   23,   29 }, {  -65,  142 },
    {  -21,  108 }, {   27,   19 }, {  -34,  111 }, {  -71,  174 },
    {  -30,  104 }, {   34,   12 }, { -144,  285 }, { -168,  304 },
    {   43,    6 }, { -125,  248 }, { -129,  252 },
    // Chroma 8x8
    {   -3,   84 }, {  -35,  122 }, {  -42,  111 }, {  101, -147 },
    {  -14,   87 }, {  -70,  179 }, {    6,   30 }, {    0,   64 },
    {    8,   32 }, {  -64,  156 }, {    0,   64 }, {    0,   64 },
    {  -65,  144 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    // Chroma 16x16
    {   19,   22 }, {   11,   48 }, {    9,   51 }, {  -24,   95 },
    {  -27,   82 }, {  -22,   92 }, {  -20,   89 }, {  -25,   82 },
    {  -15,   77 }, {   24,   32 }, {  -31,   76 }, {  -19,   77 },
    {  -21,   84 }, {  -23,   95 }, {  -35,  112 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    // Luma 4x4
    {    6,   67 }, {   -1,   72 }, {   -9,   84 }, {  -29,  100 },
    {    4,   65 }, {    0,   70 }, {   -7,   80 }, {  -34,  117 },
    {  -11,   79 }, {  -14,   91 }, {  -15,   93 }, {  -34,  119 },
    {   26,   26 }, {  -34,  111 }, {  -35,  126 },
    // Luma 8x8
    {    2,   69 }, {  -10,   82 }, {   -1,   57 }, {   -7,   59 },
    {   -5,   71 }, {  -13,   85 }, {  -28,  107 }, {  -29,  100 },
    {  -21,   93 }, {  -28,  108 }, {   -6,   71 }, {  -48,  136 },
    {  -35,  111 }, {  -10,   74 }, {    4,   56 }, {    5,   56 },
    // Luma 16x16
    {    0,   75 }, {    1,   59 }, {   -1,   67 }, {   -3,   67 },
    {    6,   29 }, {    2,   57 }, {   -5,   77 }, {   -5,   44 },
    {   -4,   64 }, {   -7,   78 }, {   -9,   34 }, {    2,   43 },
    {    2,   53 }, {    2,   60 }, {   -8,   83 },
    // Luma 32x32
    {   -7,   88 }, {    5,   52 }, {   -4,   74 }, {    5,   56 },
    // Chroma 4x4
    {  -15,  103 }, {   25,   19 }, {   23,   29 }, {  -65,  142 },
    {  -21,  108 }, {   27,   19 }, {  -34,  111 }, {  -71,  174 },
    {  -30,  104 }, {   34,   12 }, { -144,  285 }, { -168,  304 },
    {   43,    6 }, { -125,  248 }, { -129,  252 },
    // Chroma 8x8
    {   -3,   84 }, {  -35,  122 }, {  -42,  111 }, {  101, -147 },
    {  -14,   87 }, {  -70,  179 }, {    6,   30 }, {    0,   64 },
    {    8,   32 }, {  -64,  156 }, {    0,   64 }, {    0,   64 },
    {  -65,  144 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    // Chroma 16x16
    {   19,   22 }, {   11,   48 }, {    9,   51 }, {  -24,   95 },
    {  -27,   82 }, {  -22,   92 }, {  -20,   89 }, {  -25,   82 },
    {  -15,   77 }, {   24,   32 }, {  -31,   76 }, {  -19,   77 },
    {  -21,   84 }, {  -23,   95 }, {  -35,  112 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#endif

#if UNIFIED_SCAN
static const Short
INIT_ONE_FLAG[3][2*NUM_ONE_FLAG_CTX][2] =
{
  {
    { -9, 84 }, { -16, 60 }, { -11, 59 }, { -9, 61 }, 
    { -7, 65 }, { -5, 79 }, { 0, 47 }, { -5, 66 }, 
    { -7, 70 }, { -5, 68 }, { -7, 92 }, { -8, 79 }, 
    { -4, 74 }, { -4, 75 }, { -3, 70 }, { -6, 62 }, 
    { -12, 39 }, { -12, 48 }, { -13, 53 }, { -11, 55 }, 
    { -3, 63 }, { -2, 47 }, { -6, 59 }, { -8, 62 }, 
    { -5, 58 }, { -5, 78 }, { -12, 85 }, { -11, 79 }, 
    { -9, 75 }, { -3, 64 },
    { 3, 62 }, { 20, -11 }, { 16, 9 }, { 16, 19 }, 
    { 9, 38 }, { 4, 64 }, { 2, 44 }, { 1, 53 }, 
    { 3, 51 }, { 2, 54 }, { 10, 62 }, { -2, 60 }, 
    { 1, 62 }, { 8, 53 }, { 17, 36 }, { 0, 47 }, 
    { 17, -20 }, { 14, -7 }, { 8, 8 }, { 6, 19 }, 
    { 5, 46 }, { 11, 21 }, { 7, 32 }, { 1, 42 }, 
    { 9, 31 }, { -3, 66 }, { -2, 55 }, { 11, 31 }, 
    { -50, 138 }, { -8, 69 }
  },
  {
    { 0, 62 }, { 8, 21 }, { -2, 40 }, { -3, 43 }, 
    { -1, 47 }, { -1, 68 }, { 3, 39 }, { 1, 47 }, 
    { 4, 41 }, { 4, 42 }, { 2, 72 }, { -2, 63 }, 
    { -2, 66 }, { 2, 59 }, { 7, 48 }, { 2, 43 }, 
    { -6, 24 }, { -4, 25 }, { -4, 27 }, { -1, 27 }, 
    { 1, 54 }, { 7, 27 }, { 7, 31 }, { 9, 27 }, 
    { 5, 32 }, { -3, 70 }, { 3, 52 }, { 4, 48 }, 
    { 5, 47 }, { 3, 50 },
    { 7, 56 }, { 24, -14 }, { 17, 4 }, { 18, 9 }, 
    { 13, 23 }, { -28, 114 }, { -35, 97 }, { 13, 19 }, 
    { 9, 24 }, { -3, 55 }, { -18, 107 }, { 9, 45 }, 
    { 8, 36 }, { 17, 16 }, { -12, 100 }, { -7, 62 }, 
    { 11, 8 }, { -16, 54 }, { 23, -25 }, { 28, -27 }, 
    { 7, 42 }, { -38, 104 }, { 15, 24 }, { 12, 27 }, 
    { 11, 14 }, { 2, 60 }, { 27, 4 }, { 61, -73 }, 
    { 61, -73 }, { 24, 10 }
  },
  {
    { 0, 62 }, { 8, 21 }, { -2, 40 }, { -3, 43 }, 
    { -1, 47 }, { -1, 68 }, { 3, 39 }, { 1, 47 }, 
    { 4, 41 }, { 4, 42 }, { 2, 72 }, { -2, 63 }, 
    { -2, 66 }, { 2, 59 }, { 7, 48 }, { 2, 43 }, 
    { -6, 24 }, { -4, 25 }, { -4, 27 }, { -1, 27 }, 
    { 1, 54 }, { 7, 27 }, { 7, 31 }, { 9, 27 }, 
    { 5, 32 }, { -3, 70 }, { 3, 52 }, { 4, 48 }, 
    { 5, 47 }, { 3, 50 },
    { 7, 56 }, { 24, -14 }, { 17, 4 }, { 18, 9 }, 
    { 13, 23 }, { -28, 114 }, { -35, 97 }, { 13, 19 }, 
    { 9, 24 }, { -3, 55 }, { -18, 107 }, { 9, 45 }, 
    { 8, 36 }, { 17, 16 }, { -12, 100 }, { -7, 62 }, 
    { 11, 8 }, { -16, 54 }, { 23, -25 }, { 28, -27 }, 
    { 7, 42 }, { -38, 104 }, { 15, 24 }, { 12, 27 }, 
    { 11, 14 }, { 2, 60 }, { 27, 4 }, { 61, -73 }, 
    { 61, -73 }, { 24, 10 }
  }
};

static const Short
INIT_ABS_FLAG[3][2*NUM_ABS_FLAG_CTX][2] =
{
  {
    { -11, 68 }, { -4, 67 }, { -4, 73 }, { -3, 75 }, 
    { -5, 87 }, { -6, 66 }, { -4, 68 }, { -5, 74 }, 
    { -3, 73 }, { -3, 83 }, { -7, 79 }, { -6, 81 }, 
    { -7, 84 }, { -7, 85 }, { -8, 99 }, { -9, 50 }, 
    { -5, 56 }, { -4, 60 }, { -3, 62 }, { -6, 73 }, 
    { -2, 52 }, { -1, 54 }, { -1, 58 }, { 1, 55 }, 
    { -2, 65 }, { -6, 73 }, { -2, 66 }, { 0, 65 }, 
    { 0, 65 }, { -2, 75 }, 
    { -5, 53 }, { 1, 58 }, { 4, 61 }, { 5, 64 }, 
    { 3, 78 }, { 0, 55 }, { 2, 58 }, { 1, 63 }, 
    { 4, 62 }, { 2, 78 }, { 5, 53 }, { 4, 62 }, 
    { 2, 65 }, { 1, 68 }, { 4, 78 }, { -11, 47 }, 
    { -1, 43 }, { 2, 46 }, { 1, 51 }, { -2, 60 }, 
    { 5, 35 }, { 4, 46 }, { 6, 43 }, { 12, 36 }, 
    { 9, 44 }, { 5, 45 }, { -54, 149 }, { 1, 58 }, 
    { -4, 70 }, { -60, 169 }
  },
  {
    { -18, 71 }, { -7, 66 }, { -3, 68 }, { 1, 66 }, 
    { 2, 72 }, { -6, 59 }, { -4, 62 }, { -2, 64 }, 
    { -1, 66 }, { -1, 78 }, { -2, 63 }, { -3, 68 }, 
    { 0, 64 }, { 5, 58 }, { 4, 72 }, { -7, 43 }, 
    { -1, 46 }, { -1, 51 }, { 0, 55 }, { 1, 56 }, 
    { -4, 51 }, { -1, 54 }, { -5, 63 }, { -10, 74 }, 
    { 2, 56 }, { -2, 58 }, { 4, 52 }, { 2, 57 }, 
    { 2, 57 }, { -1, 67 },
    { 6, 31 }, { 2, 56 }, { 4, 63 }, { -1, 76 }, 
    { -4, 91 }, { -52, 136 }, { 3, 53 }, { -2, 64 }, 
    { -6, 80 }, { -26, 122 }, { 3, 58 }, { -1, 66 }, 
    { 4, 54 }, { -7, 79 }, { -26, 124 }, { -51, 122 }, 
    { -46, 123 }, { 13, 6 }, { 34, -22 }, { -28, 137 }, 
    { 18, 17 }, { 6, 45 }, { 33, 12 }, { -12, 102 }, 
    { 8, 47 }, { 61, -73 }, { 0, 64 }, { 0, 64 }, 
    { 61, -73 }, { 0, 64 }
  },
  {
    { -18, 71 }, { -7, 66 }, { -3, 68 }, { 1, 66 }, 
    { 2, 72 }, { -6, 59 }, { -4, 62 }, { -2, 64 }, 
    { -1, 66 }, { -1, 78 }, { -2, 63 }, { -3, 68 }, 
    { 0, 64 }, { 5, 58 }, { 4, 72 }, { -7, 43 }, 
    { -1, 46 }, { -1, 51 }, { 0, 55 }, { 1, 56 }, 
    { -4, 51 }, { -1, 54 }, { -5, 63 }, { -10, 74 }, 
    { 2, 56 }, { -2, 58 }, { 4, 52 }, { 2, 57 }, 
    { 2, 57 }, { -1, 67 },
    { 6, 31 }, { 2, 56 }, { 4, 63 }, { -1, 76 }, 
    { -4, 91 }, { -52, 136 }, { 3, 53 }, { -2, 64 }, 
    { -6, 80 }, { -26, 122 }, { 3, 58 }, { -1, 66 }, 
    { 4, 54 }, { -7, 79 }, { -26, 124 }, { -51, 122 }, 
    { -46, 123 }, { 13, 6 }, { 34, -22 }, { -28, 137 }, 
    { 18, 17 }, { 6, 45 }, { 33, 12 }, { -12, 102 }, 
    { 8, 47 }, { 61, -73 }, { 0, 64 }, { 0, 64 }, 
    { 61, -73 }, { 0, 64 }
  }
};
#else
static const Short
INIT_ONE_FLAG[3][2*NUM_ONE_FLAG_CTX][2] =
{
  {
    {  -11,   87 }, {  -20,   64 }, {  -16,   68 }, {  -13,   71 }, 
    {  -10,   73 }, {   -5,   67 }, {   -8,   26 }, {   -8,   37 }, 
    {   -3,   36 }, {   -9,   56 }, {    0,   63 }, {   -5,   39 }, 
    {  -12,   56 }, {   -9,   57 }, {   -1,   52 }, {   -4,   72 }, 
    {  -19,   73 }, {  -28,   88 }, {  -23,   85 }, {   -3,   59 }, 
    {   -2,   72 }, {  -27,   97 }, {  -22,   89 }, {  -14,   77 }, 
    {   -1,   58 }, {  -10,   86 }, {  -22,   74 }, {  -13,   63 }, 
    {   -6,   57 }, {   -5,   63 }, 
    {   -4,   70 }, {   16,   -2 }, {    9,   23 }, {    5,   41 }, 
    {   -7,   67 }, {   -6,   63 }, {   13,   -5 }, {  -12,   42 }, 
    {  -18,   53 }, {  -14,   59 }, {   -5,   65 }, {   -8,   36 }, 
    {  -22,   67 }, {  -35,   89 }, {   -3,   47 }, {  -12,   77 }, 
    {  -20,   66 }, {  -51,  113 }, {  -44,  109 }, {  -23,   84 }, 
    {   -1,   64 }, {  -58,  127 }, {  -71,  143 }, {  -67,  134 }, 
    {  -91,  187 }, {   -1,   64 }, {   20,   -3 }, {   13,   21 }, 
    {    2,   46 }, {    5,   48 } 
  },
  {
    {   -4,   71 }, {   29,   -5 }, {    1,   45 }, {   -6,   58 }, 
    {   -9,   67 }, {   -6,   66 }, {   23,   -7 }, {    3,   26 }, 
    {   -5,   42 }, {    5,   31 }, {  -12,   79 }, {   -2,   45 }, 
    {  -15,   67 }, {  -11,   62 }, {   -3,   54 }, {   -4,   70 }, 
    {  -13,   68 }, {  -31,  100 }, {  -26,   91 }, {  -18,   84 }, 
    {   -9,   83 }, {  -30,  103 }, {  -24,   90 }, {  -43,  122 }, 
    {   -6,   66 }, {    4,   59 }, {   34,  -11 }, {   21,   10 }, 
    {   14,   25 }, {    5,   44 }, 
    {   -4,   69 }, {   40,  -33 }, {   19,    8 }, {  -11,   65 }, 
    {   -6,   59 }, {  -11,   68 }, {   12,   -2 }, {   17,  -10 }, 
    {   -6,   34 }, {   11,   14 }, {  -11,   70 }, {    1,   27 }, 
    {  -26,   71 }, {  -28,   79 }, {   -2,   47 }, {    6,   47 }, 
    {  -23,   67 }, {  -47,  107 }, {  -55,  117 }, {  -21,   83 }, 
    {    1,   59 }, {  -38,   82 }, {  -34,   77 }, {  -45,   95 }, 
    {   10,   25 }, {    9,   46 }, {   41,  -31 }, {   32,  -17 }, 
    {   14,   19 }, {   17,   18 } 
  },
  {
    {   -2,   65 }, {   18,   22 }, {   10,   33 }, {   -2,   55 }, 
    {   -7,   64 }, {   -6,   67 }, {   19,   11 }, {   -5,   50 }, 
    {   -7,   53 }, {   -4,   54 }, {  -23,   99 }, {   -3,   51 }, 
    {    2,   41 }, {  -32,  102 }, {  -16,   79 }, {   -8,   77 }, 
    {  -21,   84 }, {  -26,   91 }, {  -33,  104 }, {   -4,   61 }, 
    {  -31,  122 }, {  -34,  110 }, {  -25,   96 }, {  -43,  124 }, 
    {   -6,   70 }, {    3,   60 }, {   23,   12 }, {   12,   30 }, 
    {   11,   33 }, {    8,   40 }, 
    {   -2,   66 }, {   40,  -20 }, {    0,   46 }, {   -5,   54 }, 
    {    7,   37 }, {  -39,  116 }, {   31,  -27 }, {    1,   22 }, 
    {  -35,   82 }, {  -32,   85 }, {  -15,   72 }, {   16,    0 }, 
    {  -43,  102 }, {  -75,  152 }, {   -8,   55 }, {   -9,   68 }, 
    {  -12,   54 }, {  -84,  171 }, {  -93,  186 }, {   25,   12 }, 
    { -104,  222 }, {  -40,   92 }, {  -51,   93 }, {  110, -169 }, 
    {    3,   52 }, {   17,   33 }, {   55,  -45 }, {   28,    1 }, 
    {   -5,   55 }, {   29,   -4 } 
  }
};

static const Short
INIT_ABS_FLAG[3][2*NUM_ABS_FLAG_CTX][2] =
{
  {
    {  -12,   72 }, {  -10,   79 }, {  -11,   87 }, {  -14,   94 }, 
    {  -35,  136 }, {  -10,   58 }, {   -1,   54 }, {  -17,   86 }, 
    {   -5,   70 }, {  -22,  105 }, {  -13,   70 }, {   -2,   59 }, 
    {  -13,   81 }, {  -21,   96 }, {   -3,   73 }, {  -24,   90 }, 
    {  -19,   88 }, {   -1,   63 }, {    1,   60 }, {  -17,   97 }, 
    {   -7,   69 }, {  -20,   95 }, {  -11,   84 }, {  -27,  110 }, 
    {  -12,   96 }, {   -9,   72 }, {   -8,   76 }, {   -4,   75 }, 
    {   -3,   76 }, {   -9,   94 }, 
    {  -11,   65 }, {   -4,   66 }, {   -9,   82 }, {  -24,  107 }, 
    {  -24,  111 }, {  -18,   58 }, {  -10,   61 }, {  -19,   82 }, 
    {  -28,   97 }, {    6,   47 }, {  -12,   50 }, {  -17,   73 }, 
    {   -8,   66 }, {  -44,  115 }, {  -17,   86 }, {  -26,   74 }, 
    {   -4,   48 }, {  -43,  115 }, {  -58,  141 }, {  -51,  137 }, 
    {  -51,  117 }, {  -85,  176 }, {  -47,  120 }, {  -95,  202 }, 
    {  -65,  159 }, {   -2,   53 }, {    7,   43 }, {   -2,   67 }, 
    {    3,   62 }, {    4,   67 } 
  },
  {
    {   -2,   49 }, {  -13,   81 }, {  -11,   82 }, {  -13,   88 }, 
    {  -24,  112 }, {   -4,   44 }, {  -17,   77 }, {  -16,   82 }, 
    {  -19,   89 }, {  -28,  110 }, {  -12,   64 }, {  -11,   70 }, 
    {  -18,   88 }, {   -6,   67 }, {  -19,   97 }, {  -17,   76 }, 
    {  -27,  100 }, {  -18,   88 }, {  -13,   84 }, {  -17,   94 }, 
    {  -11,   73 }, {  -15,   83 }, {  -10,   77 }, {  -10,   80 }, 
    {  -12,   91 }, {   -8,   63 }, {   -7,   71 }, {   -6,   73 }, 
    {   -8,   80 }, {   -9,   90 }, 
    {   14,   20 }, {   -7,   68 }, {   -4,   66 }, {   -8,   76 }, 
    {  -50,  148 }, {    2,   22 }, {   11,   15 }, {   -4,   49 }, 
    {  -41,  117 }, {  -60,  149 }, {  -12,   49 }, {   -6,   55 }, 
    {    1,   44 }, {   -5,   58 }, {  -12,   73 }, {   13,    9 }, 
    {    5,   26 }, {  -29,  101 }, {    0,   46 }, {  -23,   92 }, 
    {  -12,   57 }, {    6,   30 }, {   35,   -4 }, {   42,  -24 }, 
    {   22,   24 }, {   24,    0 }, {   11,   34 }, {    2,   61 }, 
    {   -5,   75 }, {    4,   62 } 
  },
  {
    {    0,   43 }, {   -6,   65 }, {   -7,   71 }, {  -31,  117 }, 
    {  -25,  109 }, {  -20,   76 }, {  -14,   73 }, {  -20,   88 }, 
    {  -21,   92 }, {   -6,   71 }, {  -19,   73 }, {  -34,  108 }, 
    {  -27,  101 }, {   -7,   69 }, {   -7,   75 }, {  -18,   77 }, 
    {   -7,   64 }, {  -20,   91 }, {   -9,   75 }, {   -7,   78 }, 
    {  -26,   98 }, {  -13,   81 }, {   -6,   69 }, {  -12,   83 }, 
    {   -2,   70 }, {   -3,   50 }, {   -6,   66 }, {   -7,   73 }, 
    {   -6,   75 }, {  -11,   91 }, 
    {   19,   10 }, {    2,   49 }, {    4,   52 }, {   -4,   69 }, 
    {  -32,  114 }, {   13,   -1 }, {  -29,   85 }, {  -64,  143 }, 
    {  -90,  186 }, {   26,    4 }, {  -24,   68 }, {    8,   30 }, 
    {  -10,   61 }, {  -16,   78 }, {  -27,   96 }, {   31,  -16 }, 
    {  -12,   54 }, {  -15,   70 }, {  -68,  158 }, {   31,   12 }, 
    {   -8,   44 }, {  -32,   63 }, {  -36,   81 }, {  -53,  106 }, 
    {   36,   12 }, {   26,   -5 }, {   19,   17 }, {  -22,  101 }, 
    {    5,   54 }, {    8,   53 } 
  }
};
#endif

// initial probability for motion vector predictor index
static const Short
INIT_MVP_IDX[3][NUM_MVP_IDX_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};

// initial probability for ALF flag
static const Short
INIT_ALF_FLAG[3][NUM_ALF_FLAG_CTX][2] =
{
  {
    {   50,  -48 }
  },
  {
    {   27,  -20 }
  },
  {
    {  -12,   68 }
  }
};

// initial probability for ALF side information (unsigned)
static const Short
INIT_ALF_UVLC[3][NUM_ALF_UVLC_CTX][2] =
{
  {
    {    1,   66 }, {   -3,   77 }
  },
  {
    {   -5,   75 }, {  -14,   94 }
  },
  {
    {   -5,   72 }, {  -30,  122 }
  }
};

// initial probability for ALF side information (signed)
static const Short
INIT_ALF_SVLC[3][NUM_ALF_SVLC_CTX][2] =
{
  {
    {   11,   57 }, {   -1,   62 }, {    0,   64 }
  },
  {
    {    6,   66 }, {   -1,   64 }, {    0,   64 }
  },
  {
    {    1,   73 }, {    2,   61 }, {    0,   64 }
  }
};
#if MTK_SAO
// initial probability for ALF flag
static const Short
INIT_AO_FLAG[3][NUM_AO_FLAG_CTX][2] =
{
  {
    {   50,  -48 }
  },
  {
    {   27,  -20 }
  },
  {
    {  -12,   68 }
  }
};

// initial probability for ALF side information (unsigned)
static const Short
INIT_AO_UVLC[3][NUM_AO_UVLC_CTX][2] =
{
  {
    {    1,   66 }, {   -3,   77 }
  },
  {
    {   -5,   75 }, {  -14,   94 }
  },
  {
    {   -5,   72 }, {  -30,  122 }
  }
};

// initial probability for ALF side information (signed)
static const Short
INIT_AO_SVLC[3][NUM_AO_SVLC_CTX][2] =
{
  {
    {   11,   57 }, {   -1,   62 }, {    0,   64 }
  },
  {
    {    6,   66 }, {   -1,   64 }, {    0,   64 }
  },
  {
    {    1,   73 }, {    2,   61 }, {    0,   64 }
  }
};
#endif

static const Short
INIT_TRANS_SUBDIV_FLAG[3][NUM_TRANS_SUBDIV_FLAG_CTX][2] =
{
  {
    {    0,    0 }, {   12,   12 }, {   22,    4 }, {   -2,   49 }, 
    {    4,   46 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   13 }, {  -28,   89 }, {  -30,   99 }, {  -34,  106 }, 
    {  -19,   76 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }
  },
  {
    {  -11,   38 }, {  -31,   88 }, {  -42,  118 }, {  -47,  130 }, 
    {  -21,   73 }, {    0,   64 }, {    0,   64 }, {    0,   64 }, 
    {    0,   64 }, {    0,   64 }
  }
};
#endif//CABAC_8BIT_INIT_TABLE2 
#endif//CABAC_8BIT_INIT_TABLE1
//! \}

#endif

